/*
 * Name:    mp_mcai.c
 *
 * Purpose: Python module interface to MX multichannel analog input functions.
 *
 * Author:  William Lavender
 *
 *---------------------------------------------------------------------------
 *
 * Copyright 2011 Illinois Institute of Technology
 *
 * See the file "LICENSE" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 */

#include "mp.h"

#include "mx_mcai.h"

MX_EXPORT PyObject *
MpPrivate_mcai_read_cmd( PyObject *self, PyObject *args )
{
	static const char cname[] = "read";

	PyObject *channels_tuple, *channel_object;
	MX_RECORD *record;
	MX_MCAI *mcai;
	long handle;
	int parse_status;
	unsigned long i, num_channels;
	double *channel_array;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "l", &handle );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	mcai = (MX_MCAI *) record->record_class_struct;

	if ( mcai == (MX_MCAI *) NULL ) {
		MpError_SetString( MpPrivateExc_Corrupt_Data_Structure,
		"%s: The MX_MCAI pointer for MCAI '%s' is NULL.",
			cname, record->name );
		return NULL;
	}

	/* Get the values from the multichannel analog input device. */

	mx_status = mx_mcai_read( record, NULL, NULL );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	num_channels = mcai->current_num_channels;

	channel_array = mcai->channel_array;

	if ( channel_array == NULL ) {
		MpError_SetString( MpPrivateExc_Corrupt_Data_Structure,
		"%s: The channel_array pointer for MCAI '%s' is NULL.",
			cname, record->name );
		return NULL;
	}

	/* Build a tuple to contain the returned channels. */

	channels_tuple = PyTuple_New( (int) num_channels );

	if ( channels_tuple == NULL )
		return NULL;

	for ( i = 0; i < num_channels; i++ ) {

		channel_object = PyFloat_FromDouble( channel_array[i] );

		if ( channel_object == NULL ) {
			Py_DECREF( channels_tuple );
			return NULL;
		}

		PyTuple_SET_ITEM( channels_tuple, i, channel_object );
	}

	return channels_tuple;
}

