/*
 * Name:    mp_portio.c
 *
 * Purpose: Python module interface to generic MX port I/O functions.
 *
 * Author:  William Lavender
 *
 *-------------------------------------------------------------------------
 *
 * Copyright 2002, 2006, 2009, 2011 Illinois Institute of Technology
 *
 * See the file "LICENSE" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 */

#include "mp.h"

#include "mx_stdint.h"
#include "mx_portio.h"

MX_EXPORT PyObject *
MpPrivate_portio_inp8_cmd(PyObject *self, PyObject *args )
{
	PyObject *result;
	MX_RECORD *record;
	long handle;
	int parse_status;
	long port_number;
	uint8_t value;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "ll", &handle, &port_number );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	value = mx_portio_inp8( record, port_number );

	result = Py_BuildValue( "i", (int) value );

	return result;
}

MX_EXPORT PyObject *
MpPrivate_portio_inp16_cmd(PyObject *self, PyObject *args )
{
	PyObject *result;
	MX_RECORD *record;
	long handle;
	int parse_status;
	long port_number;
	uint16_t value;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "ll", &handle, &port_number );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	value = mx_portio_inp16( record, port_number );

	result = Py_BuildValue( "i", (int) value );

	return result;
}

MX_EXPORT PyObject *
MpPrivate_portio_outp8_cmd(PyObject *self, PyObject *args )
{
	MX_RECORD *record;
	long handle;
	int parse_status;
	long port_number, value;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "lll",
					&handle, &port_number, &value );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	mx_portio_outp8( record, port_number, (uint8_t) value );

	Py_INCREF(Py_None);

	return Py_None;
}

MX_EXPORT PyObject *
MpPrivate_portio_outp16_cmd(PyObject *self, PyObject *args )
{
	MX_RECORD *record;
	long handle;
	int parse_status;
	long port_number, value;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "lll",
					&handle, &port_number, &value );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	mx_portio_outp16( record, port_number, (uint16_t) value );

	Py_INCREF(Py_None);

	return Py_None;
}

