/*
 * Name:    mp_scan.c
 *
 * Purpose: Python module interface to generic MX scan functions.
 *
 * Author:  William Lavender
 *
 *-------------------------------------------------------------------------
 *
 * Copyright 2002, 2009, 2011 Illinois Institute of Technology
 *
 * See the file "LICENSE" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 */

#include "mp.h"

#include "mx_scan.h"

MX_EXPORT PyObject *
MpPrivate_scan_perform_scan_cmd( PyObject *self, PyObject *args )
{
	MX_RECORD *record;
	long handle;
	int parse_status;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "l", &handle );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	mx_status = mx_perform_scan( record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	Py_INCREF(Py_None);

	return Py_None;
}

