/*
 * Name:    mp_table.c
 *
 * Purpose: Python module interface to generic MX table functions.
 *
 * Author:  William Lavender
 *
 *---------------------------------------------------------------------------
 *
 * Copyright 2002, 2006, 2009, 2011 Illinois Institute of Technology
 *
 * See the file "LICENSE" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 */

#include "mp.h"

#include "mx_table.h"

MX_EXPORT PyObject *
MpPrivate_table_is_busy_cmd( PyObject *self, PyObject *args )
{
	PyObject *result;
	MX_RECORD *record;
	long handle, axis_id;
	int parse_status;
	mx_bool_type busy;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "ll", &handle, &axis_id );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	mx_status = mx_table_is_busy( record, axis_id, &busy );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	result = Py_BuildValue( "i", (int) busy );

	return result;
}

MX_EXPORT PyObject *
MpPrivate_table_move_absolute_cmd( PyObject *self, PyObject *args )
{
	MX_RECORD *record;
	long handle, axis_id;
	int parse_status;
	double destination;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "lld", &handle,
						&axis_id, &destination );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	mx_status = mx_table_move_absolute( record, axis_id, destination );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	Py_INCREF(Py_None);

	return Py_None;
}

MX_EXPORT PyObject *
MpPrivate_table_get_position_cmd( PyObject *self, PyObject *args )
{
	PyObject *result;
	MX_RECORD *record;
	long handle, axis_id;
	int parse_status;
	double position;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "ll", &handle, &axis_id );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	mx_status = mx_table_get_position( record, axis_id, &position );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	result = Py_BuildValue( "d", position );

	return result;
}

MX_EXPORT PyObject *
MpPrivate_table_set_position_cmd( PyObject *self, PyObject *args )
{
	MX_RECORD *record;
	long handle, axis_id;
	int parse_status;
	double set_position;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "lld", &handle,
						&axis_id, &set_position );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	mx_status = mx_table_set_position( record, axis_id, set_position );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	Py_INCREF(Py_None);

	return Py_None;
}

MX_EXPORT PyObject *
MpPrivate_table_soft_abort_cmd( PyObject *self, PyObject *args )
{
	MX_RECORD *record;
	long handle, axis_id;
	int parse_status;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "ll", &handle, &axis_id );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	mx_status = mx_table_soft_abort( record, axis_id );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	Py_INCREF(Py_None);

	return Py_None;
}

MX_EXPORT PyObject *
MpPrivate_table_immediate_abort_cmd( PyObject *self, PyObject *args )
{
	MX_RECORD *record;
	long handle, axis_id;
	int parse_status;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "ll", &handle, &axis_id );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	mx_status = mx_table_immediate_abort( record, axis_id );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	Py_INCREF(Py_None);

	return Py_None;
}

MX_EXPORT PyObject *
MpPrivate_table_positive_limit_hit_cmd( PyObject *self, PyObject *args )
{
	PyObject *result;
	MX_RECORD *record;
	long handle, axis_id;
	int parse_status;
	mx_bool_type limit_hit;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "ll", &handle, &axis_id );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	mx_status = mx_table_positive_limit_hit( record, axis_id, &limit_hit );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	result = Py_BuildValue( "i", (int) limit_hit );

	return result;
}

MX_EXPORT PyObject *
MpPrivate_table_negative_limit_hit_cmd( PyObject *self, PyObject *args )
{
	PyObject *result;
	MX_RECORD *record;
	long handle, axis_id;
	int parse_status;
	mx_bool_type limit_hit;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "ll", &handle, &axis_id );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	mx_status = mx_table_negative_limit_hit( record, axis_id, &limit_hit );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	result = Py_BuildValue( "i", (int) limit_hit );

	return result;
}

